/*
 * Copyright 2013 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.0
import QtTest 1.0
import HudClient 0.1

import "../../Hud"

Hud {
    id: hud
    height: units.gu(80)
    width: units.gu(50)

    HudClientStub {
        id: hudClientStub
    }

    UnityTestCase {
        name: "HudTest"
        when: windowShown

        function test_hud_toolbar() {
            compare(hud.toolBar.children.length, 5, "toolbar item count")

            mouseClick(hud.toolBar.children[0], 0, 0)
            compare(hudClientStub.lastExecutedToolbarItem, hudClientStub.quitToolbarItemValue(), "Click on quit")

            mouseClick(hud.toolBar.children[1], 0, 0)
            compare(hudClientStub.lastExecutedToolbarItem, hudClientStub.undoToolbarItemValue(), "Click on undo")

            mouseClick(hud.toolBar.children[2], 0, 0)
            compare(hudClientStub.lastExecutedToolbarItem, hudClientStub.helpToolbarItemValue(), "Click on help")

            mouseClick(hud.toolBar.children[3], 0, 0)
            compare(hudClientStub.lastExecutedToolbarItem, hudClientStub.fullScreenToolbarItemValue(), "Click on fullscreen")

            mouseClick(hud.toolBar.children[4], 0, 0)
            compare(hudClientStub.lastExecutedToolbarItem, hudClientStub.preferencesToolbarItemValue(), "Click on preferences")
        }

        function test_hud_appstack_static_contents() {
            compare(hud.appStack.model.count, 2, "appstack item count")

            var firstApp = hud.appStack.model.get(0)
            compare(firstApp["column_0"], "app0", "first appstack item name")
            compare(firstApp["column_1"], "help", "first appstack item icon")
            compare(firstApp["column_2"], 0, "first appstack item type")

            var secondApp = hud.appStack.model.get(1)
            compare(secondApp["column_0"], "app1", "second appstack item name")
            compare(secondApp["column_1"], "folder", "second appstack item icon")
            compare(secondApp["column_2"], 1, "second appstack item type")
        }

        function test_hud_appstack_click_app() {
            mouseClick(hud.appStack.__itemsRow.children[1], 0, 0)
            compare(hudClientStub.lastSetAppStackApplicationId, "app1", "Click on second item")
        }

        function test_hud_appstack_special_item() {
            // Wait until the flickable is properly layouted
            tryCompare(hud.appStack.__itemsRow.children[1], "x", hud.appStack.__itemsRow.children[0].width + units.gu(0.5))
            compare(hud.appStack.__specialItem.visible, false, "Special item not visible at start")

            mouseClick(hud.appStack.__itemsRow.children[0], 0, 0)

            // Check that when dragging left the special item appears at pos 0
            var start_x = units.gu(12)
            var stop_x = units.gu(5)
            mouseFlick(hud.appStack.__flickable, start_x, 0, stop_x, 0, true, false)
            tryCompare(hud.appStack.__specialItem, "visible", true)
            compare(hud.appStack.__specialItem.x, 0, "Special item on left")
            mouseRelease(hud.appStack.__flickable, stop_x, 0)
            tryCompare(hud.appStack.__specialItem, "visible", false)

            // Check that when dragging right the special item appears at pos hud.appStack.width - itemwidth
            start_x = units.gu(1)
            stop_x = units.gu(150)
            mouseFlick(hud.appStack.__flickable, start_x, 0, stop_x, 0, true, false)
            tryCompare(hud.appStack.__specialItem, "visible", true)
            compare(hud.appStack.__specialItem.x + hud.appStack.__specialItem.width, hud.appStack.width, "Special item on right")
            mouseRelease(hud.appStack.__flickable, stop_x, 0)
            tryCompare(hud.appStack.__specialItem, "visible", false)
        }

        function test_hud_results() {
            tryCompare(hud.resultList.__repeater, "count", 5)

            compare(hud.resultList.__repeater.itemAt(0).children[1].nameText, "Help", "First result name")
            compare(hud.resultList.__repeater.itemAt(0).children[1].contextSnippetText, "Get Help", "First result description")

            compare(hud.resultList.__repeater.itemAt(1).children[1].nameText, "About", "Second result name")
            compare(hud.resultList.__repeater.itemAt(1).children[1].contextSnippetText, "Show About", "Second result description")

            compare(hud.resultList.__repeater.itemAt(2).children[1].nameText, "Foo", "Third result name")
            compare(hud.resultList.__repeater.itemAt(2).children[1].contextSnippetText, "Show Foo", "Third result description")

            compare(hud.resultList.__repeater.itemAt(3).children[1].nameText, "Bar", "Fourth result name")
            compare(hud.resultList.__repeater.itemAt(3).children[1].contextSnippetText, "Show Bar", "Fourth result description")

            compare(hud.resultList.__repeater.itemAt(4).children[1].nameText, "FooBar", "Fifth result name")
            compare(hud.resultList.__repeater.itemAt(4).children[1].contextSnippetText, "Show FooBar", "Fifth result description")
        }

        function test_hud_set_query() {
            compare(hudClientStub.lastSetQuery, "", "Query should be empty at start")

            hud.searchBar.text = "Hi"
            compare(hudClientStub.lastSetQuery, "Hi", "Query should be updated")

            hud.searchBar.text = "Ho"
            compare(hudClientStub.lastSetQuery, "Ho", "Query should be updated")
        }
    }
}
